#!/usr/bin/env python
# thxs to Alexander Lingo for the framework
# https://github.com/alingo?tab=activity
import sys, getopt
import pygatt
import logging

logging.basicConfig()
logging.getLogger('pygatt').setLevel(logging.DEBUG)

class dottiDevice:
    """
    A low-level class for controlling a Dotti Bluetooth LE Smart Pixel Art Light
    """

    DOTTI_HANDLE = b'\x2a'
    DOTTI_UUIDS = {"UUID_FFF3" : "0000fff3-0000-1000-8000-00805f9b34fb",
                   "UUID_FFF4" : "0000fff4-0000-1000-8000-00805f9b34fb",
                   "UUID_FFF5" : "0000fff5-0000-1000-8000-00805f9b34fb",
                   "UUID_FFF6" : "0000fff6-0000-1000-8000-00805f9b34fb"}
    SET_FILL_COLOR_HEADER = (b'\x06', b'\x01')
    SET_PIXEL_COLOR_HEADER = (b'\x07', b'\x02')
    SHOW_IMAGE_HEADER = (b'\x06', b'\x08')
    SAVE_IMAGE_HEADER = (b'\x06', b'\x07')
    SHOW_NOTIFICATIONS_HEADER = (b'\x05', b'\x21')
    CHANGE_MODE_HEADER = (b'\x04', b'\x05')
    TIME_PRE_CMD = (b'\x02', b'\x23')
    TIME_HEADER = (b'\x06', b'\x09')
    EQUALIZER_HEADER = (b'\x07', b'\x13')
    ANIMATION_SPEED_HEADER = (b'\x04', b'\x15')
    TOGGLE_NOTIFICATIONS_HEADER = (b'\x05', b'\x0a')
    ENABLE_NOTIFICATIONS_TRUE = (b'\xff', b'\x01')
    ENABLE_NOTIFICATIONS_FALSE = (b'\x00', b'\x00')

    MODES = {'favorite-image': (b'\x00',),
             'animation': (b'\x01',),
             'clock': (b'\x02',),
             'battery': (b'\x04',)}

    IMAGE_SLOTS = {'favorite-1': {'storage_addr': (b'\x00', b'\x00', b'\x00')},
                   'favorite-2': {'storage_addr': (b'\x02', b'\x90')},
                   'favorite-3': {'storage_addr': (b'\x02', b'\xa0')},
                   'favorite-4': {'storage_addr': (b'\x02', b'\xb0')},
                   'favorite-5': {'storage_addr': (b'\x02', b'\xc0')},
                   'favorite-6': {'storage_addr': (b'\x02', b'\xd0')},
                   'favorite-7': {'storage_addr': (b'\x02', b'\xe0')},
                   'favorite-8': {'storage_addr': (b'\x02', b'\xf0')},
                   'animation-1': {'storage_addr': (b'\x01', b'\x00')},
                   'animation-2': {'storage_addr': (b'\x01', b'\x10')},
                   'animation-3': {'storage_addr': (b'\x01', b'\x20')},
                   'animation-4': {'storage_addr': (b'\x01', b'\x30')},
                   'animation-5': {'storage_addr': (b'\x01', b'\x40')},
                   'animation-6': {'storage_addr': (b'\x01', b'\x50')},
                   'animation-7': {'storage_addr': (b'\x01', b'\x60')},
                   'animation-8': {'storage_addr': (b'\x01', b'\x70')},
                   'die-1': {'storage_addr': (b'\x02', b'\x00')},
                   'die-2': {'storage_addr': (b'\x02', b'\x10')},
                   'die-3': {'storage_addr': (b'\x02', b'\x20')},
                   'die-4': {'storage_addr': (b'\x02', b'\x30')},
                   'die-5': {'storage_addr': (b'\x02', b'\x40')},
                   'die-6': {'storage_addr': (b'\x02', b'\x50')},
                   'notification-1': {'storage_addr': (b'\x00', b'\x10'), 'bitmask_bytes': (b'\x01', b'\x00')},
                   'notification-2': {'storage_addr': (b'\x00', b'\x20'), 'bitmask_bytes': (b'\x02', b'\x00')},
                   'notification-3': {'storage_addr': (b'\x00', b'\x30'), 'bitmask_bytes': (b'\x04', b'\x00')},
                   'notification-4': {'storage_addr': (b'\x00', b'\x40'), 'bitmask_bytes': (b'\x08', b'\x00')},
                   'notification-5': {'storage_addr': (b'\x00', b'\x50'), 'bitmask_bytes': (b'\x10', b'\x00')},
                   'notification-6': {'storage_addr': (b'\x00', b'\x60'), 'bitmask_bytes': (b'\x20', b'\x00')},
                   'notification-7': {'storage_addr': (b'\x00', b'\x70'), 'bitmask_bytes': (b'\x40', b'\x00')},
                   'notification-8': {'storage_addr': (b'\x00', b'\x80'), 'bitmask_bytes': (b'\x80', b'\x00')},
                   'notification-9': {'storage_addr': (b'\x00', b'\x90'), 'bitmask_bytes': (b'\x00', b'\x01')}}

    def __init__(self, bt_addr):
        """
        Initializes the Dotti device class.

        Args:
            bt_addr: Bluetooth MAC address of the Dotti device you want to control.

        Returns:
            An initialized dottiDevice object, connected to the Dotti over Bluetooth LE.
        """

        self._adapter = pygatt.backends.GATTToolBackend()
        self._adapter.start()
        self._connection = self._adapter.connect(bt_addr)
        self._handle = ord(self.DOTTI_HANDLE)
        self._bt_addr = bt_addr
        self._connection.char_write(self.DOTTI_UUIDS['UUID_FFF3'], bytearray(self.TIME_PRE_CMD))

    @classmethod
    def _int_to_chr(cls, i):
        """
        Returns the ASCII character of the integer code given.

        Args:
            i: The integer to convert to a character

        Returns:
            The character associated with that integer (Example: 'P' for 80)
        """
        return chr(i)

    @classmethod
    def _ints_to_chr(cls, i_arr):
        """
        Returns the ASCII character of the integer codes given in the passed in array.

        Args:
            i_arr: An array of integers to convert to characters

        Returns:
            An array of characters associated with the integers given (Example: ['P'] for [80])
        """

        for i in i_arr:
            i = cls._int_to_chr(i)
        return i_arr

    def _write(self, byte_arr, uuid="UUID_FFF3"):
        """
        Writes the given bytearray to the Dotti at the chosen characteristic UUID

        Args:
            byte_arr: An array of bytes to write
            uuid: The Dotti UUID to write to. Most often 'UUID_FFF3'.
        """
        #dout = "Writing: "
        #for xbyte in byte_arr:
        #    dout += "[" +  hex(xbyte) + "]"
        #print(dout)
        self._connection.char_write(self.DOTTI_UUIDS[uuid], byte_arr)

    def show_notifications(self, slot_arr):
        """
        Shows the given notification colors on the Dotti.

        Args:
            slot_arr: An array of notification slot names of images to show. Ex: ['notification-1','notification-7']
        """
        if type(slot_arr) == type(''):
            slot_arr = (slot_arr,)
        slotmask = [0,0]

        for i in slot_arr:
            slotmask[0] += ord(self.IMAGE_SLOTS[i]['bitmask_bytes'][0])
            slotmask[1] += ord(self.IMAGE_SLOTS[i]['bitmask_bytes'][1])
        slotmask = tuple(slotmask)
        b_arr = bytearray(self.SHOW_NOTIFICATIONS_HEADER + self._ints_to_chr(slotmask))
        self._write(b_arr)

    def set_fill_color(self, red, green, blue):
        """
        Sets a solid color on all pixels of the Dotti

        Args:
            red: The value of the red component of the color (0-255)
            green: The value of the green component of the color (0-255)
            blue: The value of the blue component of the color (0-255)

        """
        b_arr = bytearray(self.SET_FILL_COLOR_HEADER + self._ints_to_chr((red, green, blue)))
        self._write(b_arr)

    def set_time(self, hours, minutes, seconds):
        """
        Sets the time on the Dotti

        Args:
            hours: Integer representing the hours (0-23)
            minutes: Integer representing the minutes (0-59)
            seconds: Integer representing the seconds (0-59)

        """
        b_arr_1 = bytearray(self.TIME_PRE_CMD)
        b_arr_2 = bytearray(self.TIME_HEADER + self._ints_to_chr((hours, minutes, seconds)))
        self._write(b_arr_1)
        self._write(b_arr_2)

    def show_image(self, slot_name):
        """
        Shows the specified static image on the Dotti

        Args:
            slot_name: The name of the 'slot' the image is stored in. (Ex: 'favorite-1, 'die-3')

        """

        b_arr = bytearray(self.SHOW_IMAGE_HEADER + self.IMAGE_SLOTS[slot_name]['storage_addr'])
        self._write(b_arr)

    def save_current_image(self, slot_name):
        """
        Saves the currently displayed image to the specified image slot on the Dotti

        Args:
            slot_name: The name of the 'slot' to store the image in. (Ex: 'favorite-1, 'die-3')

        """
        b_arr = bytearray(self.SAVE_IMAGE_HEADER + self.IMAGE_SLOTS[slot_name]['storage_addr'])
        self._write(b_arr)

    def change_mode(self, mode):
        """
        Changes the current mode of the Dotti

        Args:
            mode: The name mode to switch to, such as 'favorite-image' or 'clock'.

        """
        b_arr = bytearray(self.CHANGE_MODE_HEADER + self.MODES[mode])
        self._write(b_arr)

    def set_pixel_color(self, pixel_x, pixel_y, red, green, blue):
        """
        Set the color of the specified pixel on the Dotti

        Args:
            pixel_x: The x-coordinate of the pixel (1-8)
            pixel_y: The y-coordinate of the pixel (1-8)
            red: The value of the red component of the color (0-255)
            green: The value of the green component of the color (0-255)
            blue: The value of the blue component of the color (0-255)

        Pixel co-ordinates are from top left, and go top->bottom, left->right.
        The top-left pixel's coordinates are (1,1)
        The bottom-right pixel's coordinates are (8,8)

        """

        pixel_addr = pixel_x + ((pixel_y * 8) - 8)
        b_arr = bytearray(self.SET_PIXEL_COLOR_HEADER + self._ints_to_chr((pixel_addr,)) + self._ints_to_chr((red, green, blue)))
        self._write(b_arr)

    def turn_off(self):
        """
        Turns off the Dotti display

        """
        self.set_fill_color(0, 0, 0)

    def show_equalizer_image(self, intensity, red, green, blue):
        """
        Shows an 'equalizer' frame based on the intensity and color components

        Args:
            intensity: The base 'height' of the EQ bars. (0-8)
            red: The value of the red component of the color (0-255)
            green: The value of the green component of the color (0-255)
            blue: The value of the blue component of the color (0-255)

        The Dotti will generate a 'waveform' for the EQ bars based off the color components.
        The median height will be based off the 'intensity' value.
        All pixels below the EQ waveform will be filled with the given color

        """
        b_arr = bytearray(self.EQUALIZER_HEADER + self._ints_to_chr((intensity, red, green, blue)))
        self._write(b_arr)

    def set_animation_speed(self, speed):
        """
        Set the speed of the animation on the Dotti

        Args:
            speed: The speed of the animation (1-6)

        Lower speed numbers are faster

        """
        b_arr = bytearray(self.ANIMATION_SPEED_HEADER + self._ints_to_chr((speed,)))
        self._write(b_arr)

    def enable_notifications(self):
        """
        Enables notifications on the Dotti. It will show notifications given by the 'show_notifications" command.

        """
        b_arr = bytearray(self.TOGGLE_NOTIFICATIONS_HEADER + self.ENABLE_NOTIFICATIONS_TRUE)
        self._write(b_arr)

    def disable_notifications(self):
        """
        Disables notifications on the Dotti. It will NOT show notifications given by the 'show_notifications" command.

        """
        b_arr = bytearray(self.TOGGLE_NOTIFICATIONS_HEADER + self.ENABLE_NOTIFICATIONS_FALSE)
        self._write(b_arr)

def main(argv) :
    try:
       opts, args = getopt.getopt(argv,"hd:m:",["dotti=","mode="])
    except getopt.GetoptError:
       print 'pyDotti.py -d <macadress> -m <mode>'
       sys.exit(2)
    for opt, arg in opts:
      if opt == '-h':
         print 'pyDotti.py -d <macadress> -m <mode>'
         print 'mode : favorite-image,animation,clock,battery'
         sys.exit()
      elif opt in ("-d"):
         macadress = arg
      elif opt in ("-m"):
         mode = arg
    dotti=dottiDevice(macadress)
    dotti.change_mode(mode)
if __name__ == "__main__":
   main(sys.argv[1:])
